@@ -1,14 +1,14 @@
-INCBIN(detail_css, "html/detail.css");
-INCBIN(graph_css, "html/graph.css");
-INCBIN(index_html, "html/index.html");
-INCBIN(js_d3_min_js, "html/js/d3.min.js");
-INCBIN(js_jquery_min_js, "html/js/jquery.min.js");
-INCBIN(js_moment_min_js, "html/js/moment.min.js");
-INCBIN(js_rickshaw_min_js, "html/js/rickshaw.min.js");
-INCBIN(legend_css, "html/legend.css");
-INCBIN(lines_css, "html/lines.css");
-INCBIN(local_js, "html/local.js");
-INCBIN(powerdns_logo_220px_png, "html/powerdns-logo-220px.png");
+INCBIN(detail_css, "./html/detail.css");
+INCBIN(graph_css, "./html/graph.css");
+INCBIN(index_html, "./html/index.html");
+INCBIN(js_d3_min_js, "./html/js/d3.min.js");
+INCBIN(js_jquery_min_js, "./html/js/jquery.min.js");
+INCBIN(js_moment_min_js, "./html/js/moment.min.js");
+INCBIN(js_rickshaw_min_js, "./html/js/rickshaw.min.js");
+INCBIN(legend_css, "./html/legend.css");
+INCBIN(lines_css, "./html/lines.css");
+INCBIN(local_js, "./html/local.js");
+INCBIN(powerdns_logo_220px_png, "./html/powerdns-logo-220px.png");
 map<string,string> g_urlmap={
 {"detail.css", string((const char*)gdetail_cssData, gdetail_cssSize)},
 {"graph.css", string((const char*)ggraph_cssData, ggraph_cssSize)},
